/* (C) 2021 YiRing, Inc. */
package com.yiring.app.web;

import com.yiring.app.constant.Code;
import com.yiring.app.domain.location.LocationTag;
import com.yiring.app.exception.CodeException;
import com.yiring.app.util.GeoUtils;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.criteria.*;
import javax.validation.Valid;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.experimental.FieldDefaults;
import lombok.extern.slf4j.Slf4j;
import org.locationtech.jts.geom.Point;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@Validated
@Api(tags = "Hello")
@RequestMapping("/hello/")
@RestController
public class HelloController {

    String text = "😎 Hello World";

    @GetMapping
    public Result<String> hello() {
        return Result.ok(text);
    }

    /**
     * 测试失败自定义状态信息输出
     */
    @GetMapping("fail")
    public Result<String> fail() {
        throw new CodeException(Code.FAIL);
    }

    @GetMapping("page")
    public Result<PageVo<String>> page(@Valid PageParam pageParam) {
        log.info("PageParam: {}", pageParam);

        List<String> data = Arrays.asList(text.split(" "));
        PageVo<String> vo = PageVo.build(data, data.size());
        return Result.ok(vo);
    }

    @Resource
    SimpMessagingTemplate simpMessagingTemplate;

    @GetMapping("test")
    public Result<Point> test() {
        Point point = GeoUtils.createPoint(112.1, 23.56);
        simpMessagingTemplate.convertAndSend("/topic/position", point);
        return Result.ok(point);
    }

    @Resource
    EntityManager em;

    @Data
    @AllArgsConstructor
    @FieldDefaults(level = AccessLevel.PRIVATE)
    public static class UserVo {

        Long id;
        String avatar;
        String code;
    }

    @GetMapping("test2")
    public Result<ArrayList<UserVo>> query() {
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<UserVo> cq = cb.createQuery(UserVo.class);

        Root<User> root = cq.from(User.class);
        Expression<Long> id = root.get(BasicEntity.Fields.id);
        Expression<String> avatar = root.get(User.Fields.avatar);

        // 子查询
        Subquery<String> query = cq.subquery(String.class);
        Root<LocationTag> tagRoot = query.from(LocationTag.class);
        query.select(tagRoot.get(LocationTag.Fields.code));
        query.where(cb.equal(tagRoot.get(LocationTag.Fields.user), root));

        // 构建查询字段
        cq.multiselect(id, avatar, query);

        // 查询条件
        List<Predicate> predicates = new ArrayList<>();
        // 可根据入参判断是否需要查询指定字段
        predicates.add(cb.isNotNull(root.get(User.Fields.avatar)));
        predicates.add(cb.equal(query, "BTT22222222"));
        cq.where(predicates.toArray(new Predicate[0]));

        List<UserVo> users = em.createQuery(cq).getResultList();
        return Result.ok(new ArrayList<>(users));
    }
}
